@@ -32,11 +32,13 @@ import tempfile
 import time
 import traceback
 
-import gnupg  # Provided e.g. by `pip install python-gnupg` (try with `pip2`/`pip3` if `pip` does not work)
+# Provided e.g. by `pip install python-gnupg` (try with `pip3` if `pip` does not work)
+import gnupg
 import pygit2 as git
 import requests
+import deltat
 
-VERSION = '1.0.6'
+VERSION = '1.1.0'
 
 
 class GitArgumentParser(configargparse.ArgumentParser):
@@ -48,6 +50,16 @@ class GitArgumentParser(configargparse.ArgumentParser):
     def __init__(self, *args, **kwargs):
         super(GitArgumentParser, self).__init__(*args, **kwargs)
 
+    def repo_config(self, key):
+        """`repo_config(key)` is similar to `repo.config[key]`, but `key` can
+        be a comma-separated list of keys. It returns the value of the first
+        which exists or raises `KeyError` if none is set.
+        """
+        for k in key.split(','):
+            if k in repo.config:
+                return repo.config[k]
+        raise KeyError("Key%s `%s` not in git config" % ('s' if ',' in key else "", key))
+
     def add_argument(self, *args, **kwargs):
         global repo
         if repo is None and 'gitopt' in kwargs:
@@ -64,14 +76,16 @@ class GitArgumentParser(configargparse.ArgumentParser):
                 if 'action' in kwargs and issubclass(kwargs['action'],
                                                      configargparse.Action):
                     try:
-                        val = kwargs['action'].convert_default(repo.config[gitopt])
+                        val = kwargs['action'].convert_default(
+                            self.repo_config(gitopt))
                     except AttributeError:
                         raise NotImplementedError("Custom action `%r' passed "
                                                   "to GitArgumentParser does not support "
                                                   "`convert_default()' method." % kwargs['action'])
                 else:
-                    val = repo.config[gitopt]
-                kwargs['help'] += "Defaults to '%s' from `git config %s`" % (val, gitopt)
+                    val = self.repo_config(gitopt)
+                kwargs['help'] += "Defaults to '%s' from `git config %s`" % (
+                    val, gitopt.replace(',', ' or '))
                 if 'default' in kwargs:
                     kwargs['help'] += "; fallback default: '%s'" % kwargs['default']
                 kwargs['default'] = val
@@ -115,7 +129,8 @@ class DefaultTrueIfPresent(configargparse.Action):
             try:
                 values = self.convert_default(values)
             except ValueError:
-                raise configargparse.ArgumentError(self, "Requires boolean value")
+                raise configargparse.ArgumentError(
+                    self, "Requires boolean value")
         setattr(namespace, self.dest, values)
 
     @classmethod
@@ -170,9 +185,15 @@ def get_args():
     parser.add('--server',
                default='https://gitta.zeitgitter.net',
                gitopt='timestamp.server',
-               help="""Zeitgitter server to obtain timestamp from. 'https://'
+               help="""Comma-separated list of Zeitgitter servers to obtain timestamps from. 'https://'
                    is optional. The following aliases are supported: """
                     + expanded_aliases())
+    parser.add('--interval',
+               default='0s',
+               gitopt='timestamp.interval',
+               help="""Delay between timestamping against the different
+                   timestampers. For consistent ordering of timestamps,
+                   set this to at least <maximum clock skew>+1s.""")
     parser.add('--append-branch-name',
                default=True,
                action=DefaultTrueIfPresent,
@@ -180,7 +201,15 @@ def get_args():
                gitopt='timestamp.append-branch-name',
                help="""Whether to append the branch name of the current branch
                    to the timestamp branch name, i.e., create per-branch
-                   timestamp branches. (`master` will never be appended.)""")
+                   timestamp branches. (Default branch name will never be
+                   appended.)""")
+    parser.add('--default-branch',
+                gitopt='timestamp.defaultBranch',
+                default="main,master",
+                help="""Comma-separated list of default branch names, i.e.
+                    those, where the branch name will not automatically be
+                    appended to. `git config init.defaultBranch`, if it exists,
+                    is always appended to this list.""")
     parser.add('--gnupg-home',
                gitopt='timestamp.gnupg-home',
                help="Where to store timestamper public keys")
@@ -209,6 +238,12 @@ def get_args():
                help="""Which commit-ish to timestamp. Must be a branch name
                        for branch timestamps with `--append-branch-name`""")
     arg = parser.parse_args()
+    arg.interval = deltat.parse_time(arg.interval)
+    arg.default_branch = arg.default_branch.split(',')
+    try:
+        arg.default_branch.append(repo.config['init.defaultBranch'])
+    except KeyError:
+        pass
     if arg.enable == False:
         sys.exit("Timestamping explicitely disabled")
     if arg.require_enable and arg.enable != True:
@@ -237,7 +272,8 @@ def validate_key_and_import(text, args):
         sys.exit("Invalid key returned\n"
                  "Maybe not a Zeitgitter server or ~/.gnupg permission problem")
     res = gpg.import_keys(text)
-    if res.count == 1 and not args.quiet:
+    count = res.count  # pylint: disable=maybe-no-member
+    if count == 1 and not args.quiet:
         print("Imported new key %s: %s" %
               (info[0]['keyid'], info[0]['uids'][0]))
     return (info[0]['keyid'], info[0]['uids'][0])
@@ -274,7 +310,9 @@ def get_global_config_if_possible():
             try:
                 sys.stderr.write("INFO: Creating global .gitconfig\n")
                 with open(os.path.join(
-                        git.option(git.GIT_OPT_GET_SEARCH_PATH, git.GIT_CONFIG_LEVEL_GLOBAL),
+                        git.option(  # pylint: disable=maybe-no-member
+                            git.GIT_OPT_GET_SEARCH_PATH,  # pylint: disable=maybe-no-member
+                            git.GIT_CONFIG_LEVEL_GLOBAL),  # pylint: disable=maybe-no-member
                         '.gitconfig'), 'a'):
                     pass
                 return git.Config.get_global_config()  # 3
@@ -350,7 +388,8 @@ def validate_timestamp_zone_eol(header, text, offset):
                      "(off by %d seconds compared to this computer's time; check clock)"
                      % (header, time_str(istamp), istamp - sigtime))
     except ValueError:
-        sys.exit("Returned %s timestamp '%s' is not a number" % (header, stamp))
+        sys.exit("Returned %s timestamp '%s' is not a number" %
+                 (header, stamp))
     tz = text[offset + 10:offset + 17]
     if tz != ' +0000\n':
         sys.exit("Returned %s timezone is not GMT or not at end of line,\n"
@@ -439,7 +478,9 @@ def timestamp_tag(repo, keyid, name, args):
                           }, allow_redirects=False)
         quit_if_http_error(args.server, r)
         validate_tag(r.text, commit, keyid, name, args)
-        tagid = repo.write(git.GIT_OBJ_TAG, r.text)
+        tagid = repo.write(
+            git.GIT_OBJ_TAG,  # pylint: disable=maybe-no-member
+            r.text)
         repo.create_reference('refs/tags/%s' % args.tag, tagid)
     except requests.exceptions.ConnectionError as e:
         sys.exit("Cannot connect to server: %s" % e)
@@ -489,14 +530,14 @@ def valid_name(name):
             and '..' not in name and not '\n' in name)
 
 
-def append_branch_name(repo, commit_name, branch_name):
-    """Appends current branch name if not `master`"""
+def append_branch_name(repo, commit_name, branch_name, default_branches):
+    """Appends current branch name if not the default branch"""
     explanation = "for (implicit) options `--branch` and `--append-branch-name`"
     if commit_name == 'HEAD':
         try:
             comref = repo.lookup_reference(commit_name)
             comname = comref.target
-        except git.InvalidSpecError:
+        except git.InvalidSpecError:  # pylint: disable=maybe-no-member
             # 1. If HEAD or it's target is invalid, we end up here
             sys.exit("Invalid HEAD " + explanation)
         # Two more options remain:
@@ -515,14 +556,14 @@ def append_branch_name(repo, commit_name, branch_name):
         try:
             comref = repo.lookup_reference('refs/heads/' + commit_name)
             comname = commit_name  # Branch name itself
-        except (KeyError, git.InvalidSpecError):
+        except (KeyError, git.InvalidSpecError):  # pylint: disable=maybe-no-member
             # 6. Explicit commit given, but it's neither HEAD nor tail^H^H^H^H
             # a branch: fail
             sys.exit(("%s must be a branch name " + explanation)
                      % commit_name)
 
     # Now that we know which branch to timestamp (to), construct it.
-    if comname == 'master':
+    if comname in default_branches:
         return branch_name
     else:
         extended_name = "%s-%s" % (branch_name, comname)
@@ -534,15 +575,14 @@ def append_branch_name(repo, commit_name, branch_name):
                       "source branch %s)\n" + explanation)
                      % (extended_name, branch_name, comname))
 
-
-def timestamp_branch(repo, keyid, name, args):
+def timestamp_branch(repo, keyid, name, args, first):
     """Obtain and add branch commit; create/update branch head"""
     # If the base name is already invalid, it cannot become valid by appending
     if not valid_name(args.branch):
         sys.exit("Branch name %s is not valid for timestamping" %
                  args.branch)
     if args.append_branch_name:
-        args.branch = append_branch_name(repo, args.commit, args.branch)
+        args.branch = append_branch_name(repo, args.commit, args.branch, args.default_branch)
     try:
         commit = repo.revparse_single(args.commit)
     except KeyError as e:
@@ -562,17 +602,23 @@ def timestamp_branch(repo, keyid, name, args):
         try:
             if (repo[branch_head.target].parent_ids[0] == commit.id or
                     repo[branch_head.target].parent_ids[1] == commit.id):
-                sys.exit("Already timestamped commit %s to branch %s" % (commit.id.hex, args.branch))
+                sys.exit("Already timestamped commit %s to branch %s" %
+                         (commit.id.hex, args.branch))
         except IndexError:
             pass
     except KeyError:
         pass
+    if not first:
+        time.sleep(args.interval.total_seconds())
     try:
         r = requests.post(args.server, data=data, allow_redirects=False)
         quit_if_http_error(args.server, r)
         validate_branch(r.text, keyid, name, data, args)
-        commitid = repo.write(git.GIT_OBJ_COMMIT, r.text)
-        repo.create_reference('refs/heads/' + args.branch, commitid, force=True)
+        commitid = repo.write(
+            git.GIT_OBJ_COMMIT,  # pylint: disable=maybe-no-member
+            r.text)
+        repo.create_reference('refs/heads/' + args.branch,
+                              commitid, force=True)
     except requests.exceptions.ConnectionError as e:
         sys.exit("Cannot connect to server: %s" % e)
 
@@ -583,7 +629,8 @@ def main():
     try:
         # Depending on the version of pygit2, `git.discover_repository()`
         # returns `None` or raises `KeyError`
-        path = git.discover_repository(os.getcwd())
+        path = git.discover_repository(  # pylint: disable=maybe-no-member
+            os.getcwd())
     except KeyError:
         path = None
     if path is not None:
@@ -607,18 +654,19 @@ def main():
     if args.tag is not None or args.branch is not None:
         # Single tag or branch against one timestamping server
         if ',' in args.server:
-            (server, rest) = args.server.split(',', 1)
+            (server, _) = args.server.split(',', 1)
             args.server = server
             print(f"WARNING: Cannot timestamp single tag/branch against"
-                    " multiple servers;\nonly timestamping against {server}")
+                  " multiple servers;\nonly timestamping against {server}")
         (keyid, name) = get_keyid(args)
         if args.tag:
             timestamp_tag(repo, keyid, name, args)
         else:
-            timestamp_branch(repo, keyid, name, args)
+            timestamp_branch(repo, keyid, name, args, True)
     else:
         # Automatic branch, with support for multiple timestamping servers
         success = True
+        first = True
         for server in args.server.split(','):
             if server in server_aliases:
                 server = server_aliases[server]
@@ -629,7 +677,8 @@ def main():
             args.server = server
             try:
                 (keyid, name) = get_keyid(args)
-                timestamp_branch(repo, keyid, name, args)
+                timestamp_branch(repo, keyid, name, args, first)
+                first = False # Only on successful timestamp
             except SystemExit as e:
                 sys.stderr.write(e.code + '\n')
                 success = False
